/*				conf.c					*/
/*########################################################################
#                                                                        #
#                            Total IRC Access tunnel                     #
#                    by tHE rECIdjVO <recidjvo@pkcrew.org>               #
#                         http://tiatunnel.pkcrew.org/                   #
#                                                                        #
#   version: 0.9alpha3                                                   #
########################################################################*/

#include "tiatunnel.h"

extern struct tunnel *t_data;

int parse_conf_file(char *conf_file, char verbose);

int parse_conf_file(char *conf_file, char verbose)
{
	FILE *fp = NULL;
	char line[1024];
	int entries = 0;
	struct tunnel *t_current;

	// Free memory
	free(t_data);

	// Open config file
	if((fp = fopen(conf_file, "r")) == NULL) {
		perror("fopen()");
		return(-1);
	}

	// Read how many entries are stored
	while(fgets(line, 1024, fp) > 0) {
		if((line[0] != '#') && (strlen(line) > 1)) {
			entries++;
		}
	}

	// calloc space to store tunnels data
	t_data = (struct tunnel *)calloc(sizeof(struct tunnel), entries);

	if(entries > 0) {
		if(verbose == 'y') {
			printf("Config file (%s) seems to store %d entries.\n", conf_file, entries);
		}
	} else {
		printf("No valid entries found in config file (%s).\n", conf_file);
		return(-1);
	}

	// Parse config file
	rewind(fp);
	entries = 0;

	if(verbose == 'y') {
		printf("\tTYPE\tLOCALHOST\t\tREMOTEHOST\t\tREMOTEPORT\n");
	}

	while(fgets(line, 1024, fp) > 0) {
		if(line[0] == '#' || line[0] == ' ' || line[0] == '\n') {
			continue;
		}
		t_current = t_data + entries;
		if(5 != sscanf(line, "%s %c %s %s %d",
					t_current->t_passwd,
		   				&t_current->t_type,
					t_current->t_localh,
					t_current->t_remoteh,
					&t_current->t_remotep)) {
			printf("Error parsing configuration file:\n%s\n", line);
			return(-1);
		}

		// If the entry seems to be good, save it
		if(
			(strlen(t_current->t_passwd) != 0) &&
			(	(t_current->t_type == '4')
#ifdef _HAVE_TIA_IPV6
				|| (t_current->t_type == '6')
#endif
#ifdef _HAVE_TIA_SSL
				|| (t_current->t_type == 'S')
#endif
			) &&
			(strlen(t_current->t_localh) != 0) &&
			(strlen(t_current->t_remoteh) != 0) &&
			(	(t_current->t_remotep > 0) &&
				(t_current->t_remotep < 65536)
			)
		) {
			entries++;
			if(verbose == 'y') {
				printf("\t%c\t%s\t%s\t%d\n",
					t_current->t_type,
					t_current->t_localh,
					t_current->t_remoteh,
					t_current->t_remotep
				);
			}
		}
	}

	// Close config file
	fclose(fp);

	// Return
	log("Loaded %d entries", entries);
	return(entries);
}

/*				conf.c					*/
