/*				pipe.c					*/
/*########################################################################
#                                                                        #
#                            Total IRC Access tunnel                     #
#                    by tHE rECIdjVO <recidjvo@pkcrew.org>               #
#                         http://tiatunnel.pkcrew.org/                   #
#                                                                        #
#   version: 0.9alpha3                                                   #
########################################################################*/

#define NORESOLV "ERROR :Closing link: Unable to resolve address"
#define NOCONNECT "ERROR :Closing link: Unable to connect to server"

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#ifdef _HAVE_TIA_SSL
	#include <openssl/ssl.h>
#endif

#include "tiatunnel.h"

extern int resolv4(char *hostname, struct sockaddr_in *saddr);
#ifdef _HAVE_TIA_IPV6
	extern int resolv6(char *hostname, struct sockaddr_in6 *saddr6);
#endif

extern struct tunnel *t_data;

void clearpipe(int csock, struct tunnel *t_current);

void clearpipe(int csock, struct tunnel *t_current)
{
	int tsock = 0, datalen;
	char t_type = t_current->t_type;
	struct sockaddr_in saddr4;
#ifdef _HAVE_TIA_IPV6
	struct sockaddr_in6 saddr6;
#endif
#ifdef _HAVE_TIA_SSL
	SSL_CTX *sctx = NULL;
	SSL *ssocket = NULL;
#endif

	char data[4096];
	fd_set sread;

	switch(t_type) {
		case '4':
#ifdef _HAVE_TIA_SSL
		case 'S':
#endif
			saddr4.sin_family = AF_INET;

			tsock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
			if(resolv4(t_current->t_localh, &saddr4) < 0) {
				write(csock, NORESOLV "\n", (size_t)(strlen(NORESOLV) + 1));
				shutdown(csock, 2);
				shutdown(tsock, 2);
				close(csock);
				close(tsock);
				return;
			}
			bind(tsock, (struct sockaddr *)&saddr4, (socklen_t)sizeof(saddr4));

			saddr4.sin_port = htons(t_current->t_remotep);
			if(resolv4(t_current->t_remoteh, &saddr4) < 0) {
				write(csock, NORESOLV "\n", (size_t)(strlen(NORESOLV) + 1));
				shutdown(csock, 2);
				shutdown(tsock, 2);
				close(csock);
				close(tsock);
				return;
			}
			if(connect(tsock, (struct sockaddr *)&saddr4, (socklen_t)sizeof(saddr4)) < 0) {
				write(csock, NOCONNECT "\n", (size_t)(strlen(NOCONNECT) + 1));
				shutdown(csock, 2);
				shutdown(tsock, 2);
				close(csock);
				close(tsock);
				return;
			}
			break;
#ifdef _HAVE_TIA_IPV6
		case '6':
			saddr6.sin6_family = AF_INET6;

			tsock = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP);
			if(resolv6(t_current->t_localh, &saddr6) < 0) {
				write(csock, NORESOLV "\n", (size_t)(strlen(NORESOLV) + 1));
				shutdown(csock, 2);
				shutdown(tsock, 2);
				close(csock);
				close(tsock);
                        	return;
			}
			bind(tsock, (struct sockaddr *)&saddr6, (socklen_t)sizeof(saddr6)); 
			saddr6.sin6_port = htons(t_current->t_remotep);
			if(resolv6(t_current->t_remoteh, &saddr6) < 0) {
				write(csock, NORESOLV "\n", (size_t)(strlen(NORESOLV) + 1));
				shutdown(csock, 2);
				shutdown(tsock, 2);
				close(csock);
				close(tsock);
                        	return;
			}
			if(connect(tsock, (struct sockaddr *)&saddr6, (socklen_t)sizeof(saddr6)) < 0) {
				write(csock, NOCONNECT "\n", (size_t)(strlen(NOCONNECT) + 1));
				shutdown(csock, 2);
				shutdown(tsock, 2);
				close(csock);
				close(tsock);
				return;
			}
#endif
		default:
			break;
	}

	log("Connection %s <==> %s estabilished", t_current->t_localh, t_current->t_remoteh);

	// Free memory
	free(t_data);

#ifdef _HAVE_TIA_SSL
	if(t_type == 'S') {
		SSL_library_init();
	        if((sctx = SSL_CTX_new(SSLv23_client_method())) == (SSL_CTX *)NULL) {
			return;
		}
       		if((ssocket = SSL_new(sctx)) == NULL) {
                	return;
        	}
	        if(SSL_set_fd(ssocket, tsock) == 0) {
			return;
	        }
	        if(SSL_connect(ssocket) < 0) {
			return;
        	}
	}
#endif

	while(1) {
		FD_ZERO(&sread);
		FD_SET(csock, &sread);
		FD_SET(tsock, &sread);

		select(((csock > tsock) ? csock : tsock) + 1, &sread, NULL, NULL, NULL);
		bzero(data, 4096);

#ifdef _HAVE_TIA_SSL
		if(t_type == 'S') {
	        	if(FD_ISSET(csock, &sread)) {
        			if((datalen = read(csock, data, (size_t)4096)) <= 0) {
                        	        break;
                        	}
                        	SSL_write(ssocket, data, datalen);
                	}
                	if(FD_ISSET(tsock, &sread)) {
                        	if((datalen = SSL_read(ssocket, data, 4096)) <= 0) {
                                	break;
                        	}
                        	write(csock, data, (size_t)datalen);
                	}
		} else {
#endif
		if(FD_ISSET(csock, &sread)) {
			if((datalen = read(csock, data, (size_t)4096)) <= 0) {
				break;
			}
			write(tsock, data, (size_t)datalen);
		}
		if(FD_ISSET(tsock, &sread)) {
			if((datalen = read(tsock, data, (size_t)4096)) <= 0) {
				break;
			}
			write(csock, data, (size_t)datalen);
		}

	}

#ifdef _HAVE_TIA_SSL
	}

	if(t_type == 'S') {
		SSL_shutdown(ssocket);
	}
#endif

	// Close connection
	shutdown(csock, 2);
	shutdown(tsock, 2);
	close(csock);
	close(tsock);

#ifdef _HAVE_TIA_SSL
        if(t_type == 'S') {
                SSL_free(ssocket);
		SSL_CTX_free(sctx);
        }
#endif


	return;
}

/*				pipe.c					*/
