/*				tiatunnel.c				*/
/*########################################################################
#                                                                        #
#                            Total IRC Access tunnel                     #
#                    by tHE rECIdjVO <recidjvo@pkcrew.org>               #
#                         http://tiatunnel.pkcrew.org/                   #
#                                                                        #
#   version: 0.9alpha3                                                   #
########################################################################*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "tiatunnel.h"

extern int parse_conf_file(char *conf_file, char verbose);
extern struct tunnel *auth_conn(int *csock, int entries);
extern int resolv4(char *hostname, struct sockaddr_in *saddr);
extern void usage(char *argv0);
extern void clearpipe(int csock, struct tunnel *t_current);
extern void setsignals(void);
extern void resetsignals(void);

extern char *optarg;
extern int optind, opterr, optopt;

int main(int sinto, char *sinta[]);

struct tunnel *t_data;
int entries;
char conf_file[512] = DEFAULT_CONF_FILE;

int main(int sinto, char *sinta[])
{
	char optc, verbose = 'n';
	int sock, pint, csock;
	struct sockaddr_in saddr, caddr;
	char blocalh[512] = DEFAULT_BIND_HOST;
	int blocalp = DEFAULT_BIND_PORT;
	struct tunnel *t_current = NULL;

	// Set signals handler
	setsignals();

	// Open syslog stream
	openlog(NULL, LOG_PID, LOG_DAEMON | LOG_AUTHPRIV);

	// Parsing command line for parameters
	while((optc = getopt(sinto, sinta, "c:vhl:p:")) != EOF) {
		switch(optc) {
			case 'c':
				strncpy(conf_file, optarg, (size_t)512);
				break;
			case 'v':
				verbose = 'y';
				break;
			case 'h':
				usage(sinta[0]);
				exit(0);
			case 'l':
				strncpy(blocalh, optarg, (size_t)512);
				break;
			case 'p':
				blocalp = atoi(optarg);
				break;
			default:
				break;
		}
	}

	log("Starting");

	if((entries = parse_conf_file(conf_file, verbose)) < 0) {
		printf("exiting...\n");
		exit(-1);
	}

	if((sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0) {
		perror("socket()");
		exit(-1);
	}

	saddr.sin_family = AF_INET;
	saddr.sin_port = htons(blocalp);
	if(resolv4(blocalh, &saddr) < 0) {
		printf("Unable to resolve %s\n", blocalh);
		close(sock);
		exit(-1);
	}

	pint = 1;
	if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &pint, (socklen_t)sizeof(pint)) < 0) {
		perror("setsockopt()");
		close(sock);
		exit(-1);
	}
	

	// Bind endpoint
	if(bind(sock, (struct sockaddr *)&saddr, (socklen_t)sizeof(saddr)) < 0) {
		perror("bind()");
		close(sock);
		exit(-1);
	}

	// Listen for incoming connections
	if(listen(sock, 0) < 0) {
		perror("listen()");
		close(sock);
		exit(-1);
	}

	// Going in the background
	if(fork() != 0) {
		if(verbose == 'y') {
			printf("Listening on %s %d.\nGoing in daemon land...\n", blocalh, blocalp);
		}
		exit(0);
	}

	log("Listening on %s [%d]", blocalh, blocalp);

	while(1) {
		pint = sizeof(caddr);

		// Accept incoming connections
		if((csock = accept(sock, (struct sockaddr *)&caddr, (size_t *)&pint)) < 0) {
			perror("accept()");
			close(sock);
			exit(-1);
		}
		if(fork() != 0) {
			setsid();
			resetsignals();
			shutdown(sock, 2);
			close(sock);

			// Check password
			if((t_current = auth_conn(&csock, entries)) == NULL) {
				shutdown(csock, 2);
				close(csock);
				exit(-1);
			}

			// Pipe connection
			switch(t_current->t_type) {
				case '4':
#ifdef _HAVE_TIA_IPV6
				case '6':
#endif
#ifdef _HAVE_TIA_SSL
				case 'S':
#endif
					clearpipe(csock, t_current);
				default:
					break;
			}

			// Process termination
			closelog();
			exit(0);
		}
	}

	// Quit
	exit(0);
}

/*				tiatunnel.c				*/
